from enum import Enum

from pydotdict import DotDict

from ..types import Message

CHAT_START_ID = int(2E9)


class EventType(Enum):
    MESSAGE_NEW = 'message_new'
    MESSAGE_REPLY = 'message_reply'
    MESSAGE_EDIT = 'message_edit'
    MESSAGE_EVENT = 'message_event'

    MESSAGE_TYPING_STATE = 'message_typing_state'

    MESSAGE_ALLOW = 'message_allow'

    MESSAGE_DENY = 'message_deny'

    PHOTO_NEW = 'photo_new'

    PHOTO_COMMENT_NEW = 'photo_comment_new'
    PHOTO_COMMENT_EDIT = 'photo_comment_edit'
    PHOTO_COMMENT_RESTORE = 'photo_comment_restore'

    PHOTO_COMMENT_DELETE = 'photo_comment_delete'

    AUDIO_NEW = 'audio_new'

    VIDEO_NEW = 'video_new'

    VIDEO_COMMENT_NEW = 'video_comment_new'
    VIDEO_COMMENT_EDIT = 'video_comment_edit'
    VIDEO_COMMENT_RESTORE = 'video_comment_restore'

    VIDEO_COMMENT_DELETE = 'video_comment_delete'

    WALL_POST_NEW = 'wall_post_new'
    WALL_REPOST = 'wall_repost'

    WALL_REPLY_NEW = 'wall_reply_new'
    WALL_REPLY_EDIT = 'wall_reply_edit'
    WALL_REPLY_RESTORE = 'wall_reply_restore'

    WALL_REPLY_DELETE = 'wall_reply_delete'

    BOARD_POST_NEW = 'board_post_new'
    BOARD_POST_EDIT = 'board_post_edit'
    BOARD_POST_RESTORE = 'board_post_restore'

    BOARD_POST_DELETE = 'board_post_delete'

    MARKET_COMMENT_NEW = 'market_comment_new'
    MARKET_COMMENT_EDIT = 'market_comment_edit'
    MARKET_COMMENT_RESTORE = 'market_comment_restore'

    MARKET_COMMENT_DELETE = 'market_comment_delete'

    GROUP_LEAVE = 'group_leave'

    GROUP_JOIN = 'group_join'

    USER_BLOCK = 'user_block'

    USER_UNBLOCK = 'user_unblock'

    POLL_VOTE_NEW = 'poll_vote_new'

    GROUP_OFFICERS_EDIT = 'group_officers_edit'

    GROUP_CHANGE_SETTINGS = 'group_change_settings'

    GROUP_CHANGE_PHOTO = 'group_change_photo'

    VKPAY_TRANSACTION = 'vkpay_transaction'


class Event:
    def __init__(self, vk, raw):

        self.vk = vk
        self.fields = {}

        try:
            self.type = EventType(raw['type'])
        except ValueError:
            self.type = raw['type']

        self.fields.update({'type': self.type})

        for k, v in raw['object'].items():
            self.fields.update({k: DotDict(v) if isinstance(v, dict) else v})

        self.__dict__.update(self.fields)

        self.group_id = raw['group_id']

    def __repr__(self):
        return f'<{self.__class__.__name__}({", ".join(f"{k}={v}" for k, v in self.fields.items())})>'


class MessageEvent(Event, Message):
    def __init__(self, vk, raw):
        super().__init__(vk, raw)

        self.from_user = False
        self.from_chat = False
        self.from_group = False
        self.chat_id = None

        peer_id = self.message.peer_id if hasattr(self, 'message') else self.peer_id

        if peer_id < 0:
            self.from_group = True
        elif peer_id < CHAT_START_ID:
            self.from_user = True
        else:
            self.from_chat = True
            self.chat_id = peer_id - CHAT_START_ID

    def answer(self, message: str = None, **kwargs):
        if self.from_user:
            kwargs.update(user_id=self.message.from_id)
        elif self.from_chat:
            kwargs.update(peer_id=self.message.peer_id)
        elif self.from_group:
            kwargs.update(peer_id=self.message.from_id)

        self.vk.messages_send(message=message, **kwargs)

    def reply(self, message: str = None, **kwargs):
        kwargs.update(reply_to=self.message.id)
        self.answer(message=message, **kwargs)



import json
from enum import Enum
from typing import List


class Button:
    """
    Объект описывающий одну из кнопок.
    """

    class Text:
        type = 'text'

        def __init__(self, color: str, label: str, payload: dict = None):
            self.color = color
            self.action = {
                'type': self.type,
                'label': label,
                'payload': payload
            }

    class OpenLink:
        type = 'open_link'

        def __init__(self, link: str, label: str, payload: dict = None):
            self.action = {
                'type': self.type,
                'link': link,
                'label': label,
                'payload': payload

            }

    class Location:
        type = 'location'

        def __init__(self, payload: dict):
            self.action = {
                'type': self.type,
                'payload': payload
            }

    class VKPay:
        type = 'vkpay'

        def __init__(self, payload: dict, hash: str):
            self.action = {
                'type': self.type,
                'payload': payload,
                'hash': hash
            }

    class VKApps:
        type = 'open_app'

        def __init__(self, app_id: int, owner_id: int, label: str, hash: str, payload: dict = None):
            self.action = {
                'type': self.type,
                'app_id': app_id,
                'owner_id': owner_id,
                'label': label,
                'hash': hash,
                'payload': payload
            }

    class Callback:
        type = 'callback'

        def __init__(self, color: str, label: str, payload: dict):
            self.color = color
            self.action = {
                'type': self.type,
                'payload': payload,
                'label': label
            }


class Color(str, Enum):
    PRIMARY = 'primary'
    SECONDARY = 'secondary'
    NEGATIVE = 'negative'
    POSITIVE = 'positive'


class KeyboardModelMeta(type):
    def __new__(cls, name, bases, namespace):
        if bases:
            namespace['__json__'] = construct_json(namespace)

        return super().__new__(cls, name, bases, namespace)


class KeyboardModel(metaclass=KeyboardModelMeta):
    """
    Объект описывающий клавиатуру.
    """
    inline: bool = False
    one_time: bool = False

    row1: List[Button] = None
    row2: List[Button] = None
    row3: List[Button] = None
    row4: List[Button] = None
    row5: List[Button] = None

    __json__ = None


def unpack_button(button):
    if isinstance(button, (Button.Text, Button.Callback)):
        return {
            'action': button.action,
            'color': button.color
        }
    return {
        'action': button.action
    }


def construct_json(model_dict):
    rows = [row for name, row in model_dict.items() if 'row' in name and row]
    return json.dumps({
        'inline': model_dict.get('inline'),
        'one_time': model_dict.get('one_time'),
        'buttons': [
            [unpack_button(button) for button in row] for row in rows
        ]
    })


class KeyboardMarkup:
    def __init__(self, inline: bool = False, one_time: bool = False):
        self.__current_row = 0
        self.__keyboard_dict__ = {'buttons': [[], ],
                                  'inline': inline,
                                  'one_time': one_time}

    def add_button(self, button):
        self.__keyboard_dict__['buttons'][self.__current_row].append(button)

    def add_row(self):
        self.__current_row += 1
        self.__keyboard_dict__['buttons'].append([])

    def get_json(self):
        __json__ = self.__keyboard_dict__.copy()
        __json__['buttons'] = [[unpack_button(button) for button in row] for row in __json__['buttons'] if row]
        return json.dumps(__json__)



import typing

from ..types import Event, EventType


class Chat:
    chat_id: int = None
    user_id: int = None

    @classmethod
    def get_chat_id(cls):
        return cls.chat_id

    @classmethod
    def get_user_id(cls):
        return cls.chat_id

    @classmethod
    def set(cls, chat_id: int, user_id: int):
        cls.chat_id = chat_id
        cls.user_id = user_id

    @staticmethod
    def resolve_address(event: Event) -> (typing.Union[int, None], typing.Union[int, None]):
        chat, user = None, None

        if event.type is EventType.MESSAGE_NEW:
            user = event.fields['message'].get('from_id')
            chat = event.fields['message'].get('peer_id')
        elif event.type is EventType.MESSAGE_EVENT:
            user = event.fields.get('user_id')
            chat = event.fields.get('peer_id')

        return chat, user



class UnknownError(Exception):
    pass


class UnknownMethodError(Exception):
    pass


class AuthorizationError(Exception):
    pass


class PermissionError(Exception):
    pass


class WrongRequestError(Exception):
    pass


class ServerError(Exception):
    pass


class AccessIsDeniedError(Exception):
    pass


class DeprecatedMethodError(Exception):
    pass


class InvalidParametersError(Exception):
    pass


class KeyIsNotValidError(Exception):
    pass


class VkMariaException:
    exceptions = {
        1: UnknownError,
        3: UnknownMethodError,
        5: AuthorizationError,
        7: PermissionError,
        8: WrongRequestError,
        10: ServerError,
        15: AccessIsDeniedError,
        23: DeprecatedMethodError,
        27: KeyIsNotValidError,
        100: InvalidParametersError
    }

    def __init__(self, code: int, text: str):
        if code in self.exceptions:
            raise self.exceptions[code](text)
        raise Exception(text)



import re
import typing
from abc import ABC, abstractmethod

from ..fsm import FSMContext
from ..types import Event, MessageEvent, EventType


class AbstractFilter(ABC):
    @abstractmethod
    def check(self, event: Event):
        pass


class BoundFilter(AbstractFilter):
    key: str

    @abstractmethod
    def check(self, event: Event):
        pass


class EventTypeFilter(BoundFilter):
    key = 'event_type'

    def __init__(self, event_type: EventType):
        self.event_type = event_type

    def check(self, event: Event):
        if event.type == self.event_type:
            return True
        return False


class CommandsFilter(BoundFilter):
    key = 'commands'

    def __init__(self,
                 commands: typing.List[str],
                 prefixes: str = '',
                 ignore_case: bool = False):
        self.commands = tuple(map(str.lower, commands)) if ignore_case else commands
        self.prefixes = prefixes
        self.ignore_case = ignore_case

    def check(self, event: MessageEvent):
        message_text = event.message.text.lower() if self.ignore_case else event.message.text

        if not message_text:
            return False

        full_command = message_text.split()[0]
        prefix, command = ((full_command[0], full_command[1:])
                           if self.prefixes else (None, full_command))

        if prefix and prefix not in self.prefixes:
            return False

        if command in self.commands:
            return True

        return False


class TypeFromFilter(BoundFilter):
    key = 'frm'

    def __init__(self, frm: str):
        self.frm = frm

    def check(self, event: MessageEvent):
        if getattr(event, f'from_{self.frm}'):
            return True
        return False


class RegexpFilter(BoundFilter):
    key = 'regexp'

    def __init__(self, regexp: str):
        self.regexp = regexp

    def check(self, event: MessageEvent):
        if re.search(self.regexp, event.message.text, re.IGNORECASE):
            return True
        return False


class FSMStateFilter(BoundFilter):
    key = 'state'

    def __init__(self, state: str):
        self.state = state

    def check(self, event: MessageEvent):
        if self.state == FSMContext.get_current().get_state() or self.state == '*':
            return True
        return False


class Filters:
    def __init__(self, *filters: AbstractFilter):
        self.filters: typing.Tuple[AbstractFilter] = filters

    def check_all(self, event: Event) -> bool:
        if all(filter_.check(event) for filter_ in self.filters):
            return True
        return False


class FiltersFactory:
    AVAILABLE_FILTERS = (
        EventTypeFilter,
        CommandsFilter,
        TypeFromFilter,
        RegexpFilter,
        FSMStateFilter
    )

    class UnknownFilterException(Exception):
        pass

    @classmethod
    def get_filters(cls, *custom_filters, **bound_filters):
        filters = (*(cls.get_filter_by_key(key, filter_value)
                     for key, filter_value in bound_filters.items()),
                   *(custom_filter if isinstance(custom_filter, AbstractFilter) else custom_filter()
                     for custom_filter in custom_filters))
        return filters

    @classmethod
    def get_filter_by_key(cls, key: str, filter_value):
        for FILTER_CLASS in cls.AVAILABLE_FILTERS:
            if FILTER_CLASS.key == key:
                return FILTER_CLASS(filter_value)

        raise cls.UnknownFilterException(f'Нет подходящего фильтра для `{key}`')



import typing

from .filters import Filters, FiltersFactory, AbstractFilter
from ..fsm import FSMContext
from ..types import Event


class HandlerObject:
    def __init__(self, function: callable, *filters: AbstractFilter):
        self.function = function
        self.filters = Filters(*filters)

    def test_handler(self, event: Event) -> bool:
        return self.filters.check_all(event)

    def __call__(self, *args):
        if self.function.__code__.co_argcount == 2:
            args = (args[0], FSMContext.get_current())

        return self.function(*args)


class HandlerManager:
    def __init__(self):
        self.handlers: typing.List[HandlerObject] = []

    def register_handler(self,
                         function: callable,
                         *filters,
                         **bound_filters):
        # reg_bound_filters = (FiltersFactory.get_filter_by_key(key, filter_value)
        #                      for key, filter_value in bound_filters.items())

        # reg_filters = (*reg_bound_filters, *(custom_filter() for custom_filter in filters))
        reg_filters = FiltersFactory.get_filters(*filters, **bound_filters)

        self.handlers.append(HandlerObject(function, *reg_filters))



import typing

from pydotdict import DotDict


class BaseEvent:
    type: str


class MessageInfo(DotDict):
    date: int
    from_id: int
    id: int
    out: int
    peer_id: int
    text: str
    conversation_message_id: int
    fwd_messages: list
    important: bool
    random_id: int
    attachments: list
    is_hidden: bool


class Message(BaseEvent):
    message: MessageInfo
    from_user: bool
    from_chat: bool
    from_group: bool
    chat_id: typing.Union[int, None]
    peer_id: int

    def answer(self, message: str = None, domain: str = None, lat: float = None,
               long: float = None, attachment=None, reply_to: int = None, forward_messages: typing.List[int] = None,
               forward=None, sticker_id: int = None, keyboard=None, template: typing.Dict = None, payload=None,
               content_source: typing.Dict = None, dont_parse_links: int = None, disable_mentions: int = None,
               intent: str = 'default', subscribe_id: int = None): pass

    def reply(self, message: str = None, domain: str = None, lat: float = None,
              long: float = None, attachment=None, reply_to: int = None, forward_messages: typing.List[int] = None,
              forward=None, sticker_id: int = None, keyboard=None, template: typing.Dict = None, payload=None,
              content_source: typing.Dict = None, dont_parse_links: int = None, disable_mentions: int = None,
              intent: str = 'default', subscribe_id: int = None):
        """
        Не работает в беседах из-за ограниченного апи
        """
        pass



from typing import Union, Dict, List

from pydotdict import DotDict


class MessageInfo(DotDict):
    date: int
    from_id: int
    id: int
    out: int
    peer_id: int
    text: str
    conversation_message_id: int
    fwd_messages: list
    important: bool
    random_id: int
    attachments: list
    is_hidden: bool


class Message:
    message: MessageInfo
    from_user: bool
    from_chat: bool
    from_group: bool
    chat_id: Union[int, None]
    peer_id: int = None

    def answer(self, domain: str = None,
               chat_id: int = None, message: str = None, lat: float = None, long: float = None, attachment=None,
               reply_to: int = None, forward_messages: List[int] = None, forward=None, sticker_id: int = None,
               keyboard=None, template: Dict = None, payload=None, content_source: Dict = None,
               dont_parse_links: int = None, disable_mentions: int = None, intent: str = 'default',
               subscribe_id: int = None): pass

    def reply(self): pass



from .storage import FSMContext

__all__ = ('State', 'any_state', 'StatesGroup')


class State:
    def __init__(self, state):
        self._state = state

    @property
    def state(self) -> str:
        return self._state

    def set(self):
        context = FSMContext.get_current()
        context.set_state(self._state)

    def __eq__(self, other):
        return self.state == other


any_state = State('*')


class StatesGroupMeta(type):
    def __new__(mcs, name, bases, namespace, *args, **kwargs):
        cls = super().__new__(mcs, name, bases, namespace)

        states = []
        annotations = namespace.get('__annotations__')
        if annotations:
            for prop_name, prop_cls in annotations.items():
                if issubclass(prop_cls, State):
                    state = State(prop_name)
                    states.append(state)
                    setattr(cls, prop_name, state)

        cls._states = tuple(states)
        cls._state_names = tuple(state.state for state in states)

        return cls


class StatesGroup(metaclass=StatesGroupMeta):
    @classmethod
    def next(cls):
        context = FSMContext.get_current()
        state = context.get_state()

        try:
            next_step = cls._state_names.index(state) + 1
        except ValueError:
            next_step = 0

        try:
            next_state_name = cls._states[next_step].state
        except IndexError:
            next_state_name = None

        context.set_state(next_state_name)
        return next_state_name

    @classmethod
    def previous(cls) -> str:
        context = FSMContext.get_current()
        state = context.get_state()

        try:
            previous_step = cls._state_names.index(state) - 1
        except ValueError:
            previous_step = 0

        if previous_step < 0:
            previous_state_name = None
        else:
            previous_state_name = cls._states[previous_step].state_name

        context.set_state(previous_state_name)
        return previous_state_name

    @classmethod
    def first(cls) -> str:
        context = FSMContext.get_current()
        first_step_name = cls._state_names[0]
        context.set_state(first_step_name)
        return first_step_name

    @classmethod
    def last(cls) -> str:
        context = FSMContext.get_current()
        last_step_name = cls._state_names[-1]
        context.set_state(last_step_name)
        return last_step_name

    @classmethod
    def finish(cls):
        context = FSMContext.get_current()
        context.finish()



from os import PathLike
from typing import Union, List, BinaryIO

from .utils import open_files
from ..api import Vk


class Upload:
    """
    Класс реализующий загрузку файлов на сервер вк.
    """
    def __init__(self, vk: Vk):
        self.vk = vk
        self.method = self.vk.method

    def photo(self, photo: Union[str, bytes, PathLike]):
        """Загрузка фотографии, возвращает объект для вставки в сообщение"""

        data = open_files(photo, 'photo')

        upload = self.vk.photos_get_messages_upload_server()
        response = self.method(server=upload.upload_url, group_id=upload.group_id, files=data)
        p = self.vk.photos_save_messages_photo(**response)[0]

        return f'photo{p.owner_id}_{p.id}_{p.access_key}'

    def set_chat_photo(self,
                       photo: Union[str, bytes, PathLike],
                       chat_id: int,
                       crop_x: int = None,
                       crop_y: int = None,
                       crop_width: int = None):
        """Установка обложки чата"""

        data = open_files(photo, 'photo')

        upload_url = self.vk.photos_get_chat_upload_server(
            chat_id=chat_id,
            crop_x=crop_x,
            crop_y=crop_y,
            crop_width=crop_width
        )

        response = self.method(server=upload_url, files=data)

        return self.vk.messages_set_chat_photo(file=response['response'])

    def set_group_cover_photo(self,
                              photo: Union[str, bytes, PathLike],
                              crop_x: int = None,
                              crop_y: int = None,
                              crop_x2: int = None,
                              crop_y2: int = None):
        """Загрузка и установка обложки сообщества"""

        data = open_files(photo, 'photo')

        upload_url = self.vk.photos_get_owner_cover_photo_upload_server(
            crop_x=crop_x,
            crop_y=crop_y,
            crop_x2=crop_x2,
            crop_y2=crop_y2
        )

        response = self.method(server=upload_url, files=data)

        return self.vk.photos_save_owner_cover_photo(response['hash'], response['photo'])

    def document(self,
                 document: Union[str, BinaryIO, PathLike, List[Union[str, BinaryIO, PathLike]]],
                 peer_id: int,
                 title: str = None,
                 tags: List[str] = None,
                 return_tags: int = None,
                 type: str = 'doc'):
        """Загрузка документа, возвращает объект для вставки в сообщение"""

        if tags:
            tags = ','.join(tags)

        data = open_files(document, 'file')

        upload_url = self.vk.docs_get_messages_upload_server(peer_id=peer_id, type=type)
        response = self.method(server=upload_url, files=data)
        d = self.vk.docs_save(file=response['file'], title=title, tags=tags, return_tags=return_tags)

        return f'{d.type}{d.doc["owner_id"]}_{d.doc["id"]}'



from typing import *


class Adresses(TypedDict):
    is_enabled: bool
    main_address_id: int


class BanInfo(TypedDict):
    end_date: int
    comment: str


class City(TypedDict):
    id: int
    title: str


class Contacts(TypedDict):
    user_id: int
    desc: str
    phone: str
    email: str


class Counters(TypedDict):
    pass


class Country(TypedDict):
    id: int
    title: str


class Images(TypedDict):
    url: str
    width: int
    height: int


class Cover(TypedDict):
    enabled: int
    images: List[Images]


class Links(TypedDict):
    id: int
    url: str
    name: str
    desc: str
    photo_50: str
    photo_100: str


class Currency(TypedDict):
    id: int
    name: str


class Market(TypedDict):
    enabled: int
    type: str
    price_min: int
    price_max: int
    main_album_id: int
    contact_id: int
    currency: Currency
    currency_text: str


class Place(TypedDict):
    id: int
    title: str
    latitude: float
    longitude: float
    type: str
    country: int
    city: int
    address: str


class Group:
    id: int
    name: str
    screen_name: str
    is_closed: int
    deactivated: str
    is_admin: int
    admin_level: int
    is_member: int
    is_advertiser: int
    invited_by: int
    type: str
    photo_50: str
    photo_100: str
    photo_200: str
    activity: str = None
    addresses: Adresses = None
    age_limits: int = None
    ban_info: BanInfo = None
    can_create_topic: int = None
    can_message: int = None
    can_post: int = None
    can_see_all_posts: int = None
    can_upload_doc: int = None
    can_upload_video: int = None
    city: City = None
    contacts: Contacts = None
    counters: Counters = None
    country: Country = None
    cover: Cover = None
    crop_photo: Dict = None
    description: str = None
    fixed_post: int = None
    has_photo: int = None
    is_favorite: int = None
    is_hidden_from_feed: int = None
    is_messages_blocked: int = None
    links: Links = None
    main_album_id: int = None
    main_section: int = None
    market: Market = None
    member_status: int = None
    members_count: int = None
    place: Place = None
    public_date_label: str = None
    site: str = None
    start_date: int = None
    finish_date: int = None
    status: str = None
    trending: int = None
    verified: int = None
    wall: int = None
    wiki_page: str = None


class Career(TypedDict):
    group_id: int
    company: str
    country_id: int
    city_id: int
    city_name: str
    from_: int
    until: int
    position: str


class City(TypedDict):
    id: int
    title: str


class Contacts(TypedDict):
    mobile_phone: str
    home_phone: str


class Counters(TypedDict):
    albums: int
    videos: int
    audios: int
    photos: int
    notes: int
    friends: int
    groups: int
    online_friends: int
    mutual_friends: int
    user_videos: int
    followers: int
    pages: int


class Country(TypedDict):
    id: int
    title: str


class Education(TypedDict):
    university: int
    university_name: str
    faculty: int
    faculty_name: str
    graduation: int


class LastSeen(TypedDict):
    time: int
    platform: int


class Military(TypedDict):
    unit: str
    unit_id: int
    country_id: int
    from_: int
    until: int


class Occupation(TypedDict):
    type: str
    id: int
    name: str


class Personal(TypedDict):
    political: int
    langs: List[str]
    religion: str
    inspired_by: str
    people_main: int
    life_main: int
    smoking: int
    alcohol: int


class Relatives(TypedDict):
    id: int
    name: str
    type: str


class Schools(TypedDict):
    id: int
    country: int
    city: int
    name: str
    year_from: int
    year_to: int
    year_graduated: int
    class_: str
    speciality: str
    type: int
    type_str: str


class Universities(TypedDict):
    id: int
    country: int
    city: int
    name: str
    faculty: int
    faculty_name: str
    chair: int
    chair_name: str
    graduation: int
    education_form: str
    education_status: str


class Profile:
    id: int
    first_name: str
    last_name: str
    deactivated: str
    is_closed: bool
    can_access_closed: bool
    about: str = None
    activities: str = None
    bdate: str = None
    blacklisted: int = None
    blacklisted_by_me: int = None
    books: str = None
    can_post: int = None
    can_see_all_posts: int = None
    can_see_audio: int = None
    can_send_friend_request: int = None
    can_write_private_message: int = None
    career: Career = None
    city: City = None
    common_count: int = None
    connections: str = None
    contacts: Contacts = None
    counters: Counters = None
    country: Country = None
    crop_photo: dict = None
    domain: str = None
    education: Education = None
    exports = None
    first_name_nom: str = None
    first_name_gen: str = None
    first_name_dat: str = None
    first_name_acc: str = None
    first_name_ins: str = None
    first_name_abl: str = None
    followers_count: int = None
    friend_status: int = None
    games: str = None
    has_mobile: int = None
    has_photo: int = None
    home_town: str = None
    interests: str = None
    is_favorite: int = None
    is_friend: int = None
    is_hidden_from_feed: int = None
    last_name_nom: str = None
    last_name_gen: str = None
    last_name_dat: str = None
    last_name_acc: str = None
    last_name_ins: str = None
    last_name_abl: str = None
    last_seen: LastSeen = None
    lists: str = None
    maiden_name: str = None
    military: Military = None
    movies: str = None
    music: str = None
    nickname: str = None
    occupation: Occupation = None
    online: int = None
    personal: Personal = None
    photo_50: str = None
    photo_100: str = None
    photo_200_orig: str = None
    photo_200: str = None
    photo_400_orig: str = None
    photo_id: str = None
    photo_max: str = None
    photo_max_orig: str = None
    quotes: str = None
    relatives: Relatives = None
    relation: int = None
    schools: Schools = None
    screen_name: str = None
    sex: int = None
    site: str = None
    status: str = None
    timezone: int = None
    trending: int = None
    tv: str = None
    universities: Universities = None
    verified: int = None
    wall_default: str = None


class BanInfo:
    admin_id: int
    date: int
    reason: int
    comment: str
    end_date: int


class Order:
    id: int
    group_id: int
    user_id: int
    date: int
    variants_grouping_id: int
    is_main_variant: bool
    property_values: List
    cart_quantity: int
    status: int
    items_count: int
    total_price: Dict
    display_order_id: str
    comment: str
    preview_order_items: List
    delivery: Dict
    recipient: Dict


class Document:
    id: int
    owner_id: int
    title: str
    size: int
    ext: str
    url: str
    date: int
    type: int
    preview: Dict



from _io import BufferedReader
from typing import Union, List, BinaryIO

from .exceptions import *
from .. import working_dir


def open_file(file: Union[str, BinaryIO]):

    if isinstance(file, BufferedReader):
        file = file
    elif isinstance(file, str):
        file = open(working_dir + '/' + file, 'rb')
    else:
        raise InvalidFileFormatError('Неверный формат файла')

    return file


def open_files(files: Union[str, BinaryIO, List[Union[str, BinaryIO]]], type: str):

    if isinstance(files, list) and len(files) > 1:
        data = {f'{type}{i}': open_file(f) for i, f in enumerate(files, start=1)}
    else:
        data = {type: open_file(files)}

    return data
